"""JARVIS database models."""
from datetime import datetime
from typing import Optional

from beanie import Document, Link
from pydantic import BaseModel, Field

from jarvis_core.db.models.actions import Ban, Kick, Mute, Unban, Warning
from jarvis_core.db.models.captcha import Captcha
from jarvis_core.db.models.modlog import Action, Modlog, Note
from jarvis_core.db.models.reddit import Subreddit, SubredditFollow
from jarvis_core.db.models.twitter import TwitterAccount, TwitterFollow
from jarvis_core.db.utils import NowField, Snowflake, SnowflakeDocument

__all__ = [
    "Action",
    "Autopurge",
    "Autoreact",
    "Ban",
    "Captcha" "Config",
    "Filter",
    "Guess",
    "Kick",
    "Lock",
    "Lockdown",
    "Modlog",
    "Mute",
    "Note",
    "Pin",
    "Pinboard",
    "Phishlist",
    "Purge",
    "Reminder",
    "Rolegiver",
    "Bypass",
    "Roleping",
    "Setting",
    "Subreddit",
    "SubredditFollow",
    "Tag",
    "Temprole",
    "TwitterAccount",
    "TwitterFollow",
    "Unban",
    "UserSetting",
    "Warning",
    "all_models",
]


class Autopurge(SnowflakeDocument):
    guild: Snowflake
    channel: Snowflake
    delay: int = 30
    admin: Snowflake
    created_at: datetime = NowField()


class Autoreact(SnowflakeDocument):
    guild: Snowflake
    channel: Snowflake
    reactions: list[str] = Field(default_factory=list)
    admin: Snowflake
    thread: bool
    created_at: datetime = NowField()


class Config(SnowflakeDocument):
    """Config database object."""

    key: str
    value: str | int | bool


class Filter(SnowflakeDocument):
    """Filter database object."""

    guild: Snowflake
    name: str
    filters: list[str] = Field(default_factory=list)


class Guess(SnowflakeDocument):
    """Guess database object."""

    correct: bool
    guess: str
    user: Snowflake


class Permission(BaseModel):
    """Embedded Permissions document."""

    id: Snowflake
    allow: Optional[Snowflake] = 0
    deny: Optional[Snowflake] = 0


class Lock(SnowflakeDocument):
    """Lock database object."""

    active: bool = True
    admin: Snowflake
    channel: Snowflake
    duration: int = 10
    reason: str
    original_perms: Permission
    created_at: datetime = NowField()


class Lockdown(SnowflakeDocument):
    """Lockdown database object."""

    active: bool = True
    admin: Snowflake
    duration: int = 10
    guild: Snowflake
    reason: str
    original_perms: Snowflake
    created_at: datetime = NowField()


class Purge(SnowflakeDocument):
    """Purge database object."""

    admin: Snowflake
    channel: Snowflake
    guild: Snowflake
    count_: int = Field(10, alias="count")
    created_at: datetime = NowField()


class Reminder(SnowflakeDocument):
    """Reminder database object."""

    active: bool = True
    user: Snowflake
    guild: Snowflake
    channel: Snowflake
    message: str
    remind_at: datetime
    created_at: datetime = NowField()
    repeat: Optional[str] = None
    timezone: str = "UTC"
    total_reminders: int = 0
    parent: Optional[str] = None
    private: bool = False


class Rolegiver(SnowflakeDocument):
    """Rolegiver database object."""

    guild: Snowflake
    roles: Optional[list[Snowflake]] = Field(default_factory=list)
    group: Optional[str] = None


class Bypass(BaseModel):
    """Roleping bypass embedded object."""

    users: Optional[list[Snowflake]] = Field(default_factory=list)
    roles: Optional[list[Snowflake]] = Field(default_factory=list)


class Roleping(SnowflakeDocument):
    """Roleping database object."""

    active: bool = True
    role: Snowflake
    guild: Snowflake
    admin: Snowflake
    bypass: Bypass
    created_at: datetime = NowField()


class Setting(SnowflakeDocument):
    """Setting database object."""

    guild: Snowflake
    setting: str
    value: str | int | bool | list[int | str]


class Phishlist(SnowflakeDocument):
    """Phishlist database object."""

    url: str
    confirmed: bool = False
    valid: bool = True
    created_at: datetime = NowField()


class Pinboard(SnowflakeDocument):
    """Pinboard database object."""

    channel: Snowflake
    guild: Snowflake
    admin: Snowflake
    created_at: datetime = NowField()


class Pin(SnowflakeDocument):
    """Pin database object."""

    active: bool = True
    index: int
    message: Snowflake
    channel: Snowflake
    pinboard: Link[Pinboard]
    guild: Snowflake
    admin: Snowflake
    pin: Snowflake
    created_at: datetime = NowField()


class Tag(SnowflakeDocument):
    """Tag database object."""

    creator: Snowflake
    name: str
    content: str
    guild: Snowflake
    created_at: datetime = NowField()
    edited_at: Optional[datetime] = None
    editor: Optional[Snowflake] = None


class Temprole(SnowflakeDocument):
    """Temporary role object."""

    guild: Snowflake
    user: Snowflake
    role: Snowflake
    admin: Snowflake
    expires_at: datetime
    reapply_on_rejoin: bool = True
    created_at: datetime = NowField()


class UserSetting(SnowflakeDocument):
    """User Setting object."""

    user: Snowflake
    type: str
    setting: str
    value: str | int | bool


all_models: list[Document] = [
    Autopurge,
    Autoreact,
    Ban,
    Captcha,
    Config,
    Filter,
    Guess,
    Kick,
    Lock,
    Lockdown,
    Modlog,
    Mute,
    Pin,
    Pinboard,
    Phishlist,
    Purge,
    Reminder,
    Rolegiver,
    Roleping,
    Setting,
    Subreddit,
    SubredditFollow,
    Tag,
    Temprole,
    TwitterAccount,
    TwitterFollow,
    Unban,
    UserSetting,
    Warning,
]
